@@ -38,8 +38,19 @@ psychonetrics_fitfunction <- function(x, model){
       "FIML" = switch(distribution,"Gaussian" = fimlEstimator_Gauss)
     )
   }
-  
-  
-  # Run and return:
-  estFun(prep)
+
+  # PML/PFIML use dedicated penalized fit functions (handles cpp internally):
+  if (estimator == "PML") {
+    fit <- switch(distribution,
+                  "Gaussian" = penMaxLikEstimator_Gauss(prep, x, model),
+                  "Ising" = penMaxLikEstimator_Ising(prep, x, model))
+  } else if (estimator == "PFIML") {
+    fit <- switch(distribution,
+                  "Gaussian" = penFIMLEstimator_Gauss(prep, x, model))
+  } else {
+    # Run standard estimator:
+    fit <- estFun(prep)
+  }
+
+  fit
 }
\ No newline at end of file
