@@ -14,8 +14,10 @@ impliedModel <- function(model, types, all = FALSE){
                      "tsdlvm1" = implied_tsdlvm1_cpp, # <- Updated!
                      "meta_varcov" = implied_meta_varcov_cpp, # <- Updated!
                      "Ising" = implied_Ising_cpp, # <- Updated!
-                     "ml_lvm" = implied_ml_lvm_cpp # <- Updated!
-    )    
+                     "ml_lvm" = implied_ml_lvm_cpp, # <- Updated!
+                     "meta_lvm" = implied_meta_lvm_cpp,
+                     "meta_var1" = implied_meta_var1_cpp
+    )
   } else {
     impFun <- switch(framework,
                      "lvm" = implied_lvm,
@@ -27,7 +29,9 @@ impliedModel <- function(model, types, all = FALSE){
                      "tsdlvm1" = implied_tsdlvm1,
                      "meta_varcov" = implied_meta_varcov,
                      "Ising" = implied_Ising,
-                     "ml_lvm" = implied_ml_lvm
+                     "ml_lvm" = implied_ml_lvm,
+                     "meta_lvm" = implied_meta_lvm,
+                     "meta_var1" = implied_meta_var1
     )
   }
 
