@@ -20,8 +20,8 @@ psychonetrics_print_logo <- function(color = c("default","red","green","yellow",
   logo <- c("                       _                      _        _          ",
             "                      | |                    | |      (_)         ",
             "  _ __  ___ _   _  ___| |__   ___  _ __   ___| |_ _ __ _  ___ ___ ",
-            " |  _ \\/ __| | | |/ __|  _ \\ / _ \\|  _ \\ / _ \\ __|  __| |/ __/ __|",
-            " | |_) \\__ \\ |_| | (__| | | | (_) | | | |  __/ |_| |  | | (__\\__ \\",
+            " |    \\/ __| | | |/ __|  _ \\ /   \\|  _ \\ / _ \\ __|  __| |/ __/ __|",
+            " |  O--------------------------O  | | | |  __/ |_| |  | | (__\\__ \\",
             " | .__/|___/\\__, |\\___|_| |_|\\___/|_| |_|\\___|\\__|_|  |_|\\___|___/",
             " | |         __/ |                                                ",
             paste0(" |_|        |___/  ",paste0(rep(" ",66 - 19 - nchar(version_string)),collapse=""),version_string),
