@@ -26,7 +26,11 @@ Ising <- function(
   min_sum = -Inf, # Used for threhsolded Ising model estimation
   bootstrap = FALSE,
   boot_sub,
-  boot_resample
+  boot_resample,
+  # Penalized ML arguments:
+  penalty_lambda = NA,  # Penalty strength (NA = auto-select via EBIC grid search)
+  penalty_alpha = 1,   # Elastic net mixing: 1 = LASSO, 0 = ridge
+  penalize_matrices  # Character vector of matrix names to penalize. Default: defaultPenalizeMatrices()
 ){
   covtype <- match.arg(covtype)
   beta_model <- match.arg(beta_model)
@@ -71,9 +75,9 @@ Ising <- function(
     # }
   }
   
-  # Fail if estimator is not ML (nothing else supported yet):
-  if (estimator != "ML"){
-    stop("Only ML estimation is currently supported for Ising model.")
+  # Fail if estimator is not ML or PML (nothing else supported yet):
+  if (!estimator %in% c("ML", "PML")){
+    stop("Only ML and PML estimation are currently supported for Ising model.")
   }
   
   # Obtain sample stats:
@@ -236,7 +240,21 @@ Ising <- function(
   } else {
     model <- setoptimizer(model, optimizer)
   }
-  
+
+  # Setup PML penalization:
+  if (estimator == "PML") {
+    model@penalty <- list(lambda = penalty_lambda, alpha = penalty_alpha)
+    pen_mats <- if (missing(penalize_matrices)) defaultPenalizeMatrices(model) else penalize_matrices
+    model <- penalize(model, matrix = pen_mats, lambda = penalty_lambda, log = FALSE)
+    # Baseline/saturated models should use ML, not PML:
+    if (!is.null(model@baseline_saturated$baseline)) {
+      model@baseline_saturated$baseline@estimator <- "ML"
+    }
+    if (!is.null(model@baseline_saturated$saturated)) {
+      model@baseline_saturated$saturated@estimator <- "ML"
+    }
+  }
+
   # Return model:
   return(model)
 }
\ No newline at end of file
