@@ -21,14 +21,17 @@ var1 <- function(
   dayvar,
   idvar,
   vars, # character indicating the variables Extracted if missing from data - group variable
-  groups, # ignored if missing. Can be character indicating groupvar, or vector with names of groups
+  groups, # deprecated, use groupvar instead
+  groupvar, # grouping variable (character column name or vector of group names)
   covs, # alternative covs (array nvar * nvar * ngroup)
+  cors, # alternative cors (treated as covs with warning in var1)
   means, # alternative means (matrix nvar * ngroup)
   nobs, # Alternative if data is missing (length ngroup)
-  missing = "listwise",
+  corinput, # not supported in var1 (errors if TRUE)
+  missing = "auto",
   equal = "none", # Can also be any of the matrices
   baseline_saturated = TRUE, # Leave to TRUE! Only used to stop recursive calls
-  # fitfunctions, 
+  # fitfunctions,
   estimator = "ML",
   optimizer,
   storedata = FALSE,
@@ -38,10 +41,32 @@ var1 <- function(
   verbose=FALSE,
   bootstrap = FALSE,
   boot_sub,
-  boot_resample
+  boot_resample,
+  # Penalized ML arguments:
+  penalty_lambda = NA,  # Penalty strength (NA = auto-select via EBIC grid search)
+  penalty_alpha = 1,   # Elastic net mixing: 1 = LASSO, 0 = ridge
+  penalize_matrices  # Character vector of matrix names to penalize. Default: defaultPenalizeMatrices()
 ){
   contemporaneous <- match.arg(contemporaneous)
-  
+
+  # Standardize input arguments:
+  si <- standardize_input(
+    data = if(missing(data)) NULL else data,
+    covs = if(missing(covs)) NULL else covs,
+    cors = if(missing(cors)) NULL else cors,
+    nobs = if(missing(nobs)) NULL else nobs,
+    corinput = if(missing(corinput)) NULL else corinput,
+    groups = if(missing(groups)) NULL else groups,
+    groupvar = if(missing(groupvar)) NULL else groupvar,
+    family = "var1", caller = "var1()", estimator = estimator
+  )
+  # Only overwrite when standardize_input actually resolved a value,
+  # so that the original missing() state is preserved for downstream code:
+  if (!is.null(si$data)) data <- si$data
+  if (!is.null(si$covs)) covs <- si$covs
+  if (!is.null(si$nobs)) nobs <- si$nobs
+  if (!is.null(si$groups)) groups <- si$groups
+
   # FIXME: Not sure why needed...
   if (missing(vars)) vars2 <- NULL else vars2 <- vars
   if (missing(idvar)) idvar <- NULL
@@ -50,7 +75,7 @@ var1 <- function(
   if (missing(groups)) groups <- NULL
   
   # If data is missing with rawts, stop:
-  if (!missing(data)){
+  if (!is.null(data)){
     data <- as.data.frame(data)
     if (is.null(names(data))){
       names(data) <- paste0("V",seq_len(ncol(data)))
@@ -72,16 +97,33 @@ var1 <- function(
     vars <- colnames(data)[colnames(data)!=groups]
   }
   
+  # Auto-detect missing data handling:
+  if (missing == "auto") {
+    has_missing <- any(is.na(data[, vars, drop = FALSE]))
+    if (has_missing) {
+      if (estimator == "ML") {
+        estimator <- "FIML"
+      } else if (estimator == "PML") {
+        estimator <- "PFIML"
+      } else {
+        # LS variants: default to listwise
+        missing <- "listwise"
+      }
+    } else {
+      missing <- "listwise"
+    }
+  }
+
   # Obtain sample stats:
   if (missing(sampleStats)){
-    sampleStats <- samplestats(data = data, 
-                               vars = vars, 
+    sampleStats <- samplestats(data = data,
+                               vars = vars,
                                groups = groups,
-                               covs = covs, 
-                               means = means, 
-                               nobs = nobs, 
-                               missing  = ifelse(estimator == "FIML","pairwise",missing),
-                               fimldata = estimator == "FIML",
+                               covs = covs,
+                               means = means,
+                               nobs = nobs,
+                               missing  = ifelse(estimator %in% c("FIML", "PFIML"),"pairwise",missing),
+                               fimldata = estimator %in% c("FIML", "PFIML"),
                                storedata = storedata,
                                covtype=covtype,
                                standardize = standardize,
@@ -300,12 +342,12 @@ var1 <- function(
     
 
     
-    # if not FIML, Treat as computed:
-    if (estimator != "FIML"){
+    # if not FIML/PFIML, Treat as computed:
+    if (!estimator %in% c("FIML", "PFIML")){
       model@baseline_saturated$saturated@computed <- TRUE
-      
+
       # FIXME: TODO
-      model@baseline_saturated$saturated@objective <- psychonetrics_fitfunction(parVector(model@baseline_saturated$saturated),model@baseline_saturated$saturated)      
+      model@baseline_saturated$saturated@objective <- psychonetrics_fitfunction(parVector(model@baseline_saturated$saturated),model@baseline_saturated$saturated)
     }
     
   }
@@ -315,7 +357,22 @@ var1 <- function(
   } else {
     model <- setoptimizer(model, optimizer)
   }
-  
+
+  # Setup PML/PFIML penalization:
+  if (estimator %in% c("PML", "PFIML")) {
+    model@penalty <- list(lambda = penalty_lambda, alpha = penalty_alpha)
+    pen_mats <- if (missing(penalize_matrices)) defaultPenalizeMatrices(model) else penalize_matrices
+    model <- penalize(model, matrix = pen_mats, lambda = penalty_lambda, log = FALSE)
+    # Baseline/saturated models should use unpenalized estimator:
+    base_est <- if (estimator == "PFIML") "FIML" else "ML"
+    if (!is.null(model@baseline_saturated$baseline)) {
+      model@baseline_saturated$baseline@estimator <- base_est
+    }
+    if (!is.null(model@baseline_saturated$saturated)) {
+      model@baseline_saturated$saturated@estimator <- base_est
+    }
+  }
+
   # Return model:
   return(model)
 }
