@@ -14,11 +14,13 @@ varcov <- function(
   tau,
   vars, # character indicating the variables Extracted if missing from data - group variable
   ordered = character(0), # character indicating the variables that are ordinal
-  groups, # ignored if missing. Can be character indicating groupvar, or vector with names of groups
+  groups, # deprecated, use groupvar instead
+  groupvar, # grouping variable (character column name or vector of group names)
   covs, # alternative covs (array nvar * nvar * ngroup)
+  cors, # alternative cors (treated as covs with corinput=TRUE for varcov)
   means, # alternative means (matrix nvar * ngroup)
   nobs, # Alternative if data is missing (length ngroup)
-  missing = "listwise",
+  missing = "auto",
   equal = "none", # Can also be any of the matrices
   baseline_saturated = TRUE, # Leave to TRUE! Only used to stop recursive calls
   estimator = "default",
@@ -34,8 +36,31 @@ varcov <- function(
   fullFIML=FALSE,
   bootstrap = FALSE,
   boot_sub,
-  boot_resample
+  boot_resample,
+  # Penalized ML arguments:
+  penalty_lambda = NA,  # Penalty strength (NA = auto-select via EBIC grid search)
+  penalty_alpha = 1,   # Elastic net mixing: 1 = LASSO, 0 = ridge
+  penalize_matrices  # Character vector of matrix names to penalize. Default: defaultPenalizeMatrices()
 ){
+  # Standardize input arguments:
+  si <- standardize_input(
+    data = if(missing(data)) NULL else data,
+    covs = if(missing(covs)) NULL else covs,
+    cors = if(missing(cors)) NULL else cors,
+    nobs = if(missing(nobs)) NULL else nobs,
+    corinput = if(missing(corinput)) NULL else corinput,
+    groups = if(missing(groups)) NULL else groups,
+    groupvar = if(missing(groupvar)) NULL else groupvar,
+    family = "varcov", caller = "varcov()", estimator = estimator
+  )
+  # Only overwrite when standardize_input actually resolved a value,
+  # so that the original missing() state is preserved for downstream code:
+  if (!is.null(si$data)) data <- si$data
+  if (!is.null(si$covs)) covs <- si$covs
+  if (!is.null(si$nobs)) nobs <- si$nobs
+  if (!is.null(si$corinput)) corinput <- si$corinput
+  if (!is.null(si$groups)) groups <- si$groups
+
   rawts = FALSE
   if (rawts){
     warning("'rawts' is only included for testing purposes. Please do not use!")
@@ -46,14 +71,24 @@ varcov <- function(
     ordered <- character(0)
   }
   
+  # WLSMV is a synonym for DWLS (DWLS estimation + scaled test statistic):
+  if (estimator == "WLSMV"){
+    estimator <- "DWLS"
+  }
+
   if (estimator == "default"){
     if (length(ordered) > 0){
-      estimator <- "WLS"
+      estimator <- "DWLS"
     } else {
       estimator <- "ML"
     }
   }
-  
+
+  # Experimental warnings:
+  if (length(ordered) > 0) {
+    experimentalWarning("ordinal data in varcov()")
+  }
+
   # Check WLS for ordinal:
   if (length(ordered) > 0 & !estimator %in% c("WLS","DWLS","ULS")){
     stop("Ordinal data is only supported for WLS, DWLS and ULS estimators.")
@@ -82,11 +117,37 @@ varcov <- function(
   }
   
   # Check FIML:
-  if (!missing(data) && !meanstructure && estimator == "FIML"){
-    stop("meanstructure = FALSE is not yet supported for 'FIML' estimator")
+  if (!missing(data) && !meanstructure && estimator %in% c("FIML", "PFIML")){
+    stop("meanstructure = FALSE is not yet supported for 'FIML'/'PFIML' estimator")
   }
-  
-  
+
+  # Auto-detect missing data handling:
+  if (missing == "auto") {
+    if (!missing(data)) {
+      if (missing(vars)) {
+        check_vars <- if (!is.null(colnames(data))) colnames(data) else seq_len(ncol(data))
+      } else {
+        check_vars <- vars
+      }
+      has_missing <- any(is.na(data[, check_vars, drop = FALSE]))
+      if (has_missing) {
+        if (estimator == "ML") {
+          estimator <- "FIML"
+        } else if (estimator == "PML") {
+          estimator <- "PFIML"
+        } else {
+          # LS variants: default to listwise (WLS weights don't support missing data for continuous)
+          missing <- "listwise"
+        }
+      } else {
+        missing <- "listwise"
+      }
+    } else {
+      # No raw data (covs/means provided): fall back to listwise
+      missing <- "listwise"
+    }
+  }
+
   # Obtain sample stats:
   if (missing(sampleStats)){
     # WLS weights:
@@ -106,9 +167,9 @@ varcov <- function(
                                covs = covs, 
                                means = means, 
                                nobs = nobs, 
-                               missing = ifelse(estimator == "FIML","pairwise",missing),
+                               missing = ifelse(estimator %in% c("FIML", "PFIML"),"pairwise",missing),
                                rawts = rawts,
-                               fimldata = estimator == "FIML",
+                               fimldata = estimator %in% c("FIML", "PFIML"),
                                storedata = storedata,
                                weightsmatrix = WLS.W,
                                meanstructure = meanstructure,
@@ -386,8 +447,8 @@ varcov <- function(
     }
 
     
-    # if not FIML, Treat as computed:
-    if (estimator != "FIML"){
+    # if not FIML/PFIML, Treat as computed:
+    if (!estimator %in% c("FIML", "PFIML")){
       model@baseline_saturated$saturated@computed <- TRUE
       
       # FIXME: TODO
@@ -401,7 +462,21 @@ varcov <- function(
     model <- setoptimizer(model, optimizer)
   }
 
-  
+  # Setup PML/PFIML penalization:
+  if (estimator %in% c("PML", "PFIML")) {
+    model@penalty <- list(lambda = penalty_lambda, alpha = penalty_alpha)
+    pen_mats <- if (missing(penalize_matrices)) defaultPenalizeMatrices(model) else penalize_matrices
+    model <- penalize(model, matrix = pen_mats, lambda = penalty_lambda, log = FALSE)
+    # Baseline/saturated models should use unpenalized estimator:
+    base_est <- if (estimator == "PFIML") "FIML" else "ML"
+    if (!is.null(model@baseline_saturated$baseline)) {
+      model@baseline_saturated$baseline@estimator <- base_est
+    }
+    if (!is.null(model@baseline_saturated$saturated)) {
+      model@baseline_saturated$saturated@estimator <- base_est
+    }
+  }
+
   # Return model:
   return(model)
 }
\ No newline at end of file
