@@ -1,6 +1,6 @@
 # this function will automatically identify models:
 identify <- function(x){
-  if (x@model == "ggm" | x@model == "precision"| x@model == "gvar" | x@model == "varcov" | x@model == "cholesky" | x@model == "var1" | x@model == "panelvar1" | x@model == "meta_varcov"){
+  if (x@model == "ggm" | x@model == "precision"| x@model == "gvar" | x@model == "varcov" | x@model == "cholesky" | x@model == "var1" | x@model == "panelvar1" | x@model == "meta_varcov" | x@model == "meta_var1"){
     # Nothing to do..
     return(x)
   }
@@ -27,6 +27,9 @@ identify <- function(x){
   } else if (x@model == "ml_lvm"){
     x <- identify_ml_lvm(x)
     return(x)
+  } else if (x@model == "meta_lvm"){
+    x <- identify_lvm(x)
+    return(x)
   }
   
   
