@@ -18,7 +18,9 @@ runmodel <- function(
     warn_bounds = TRUE,
     return_improper = TRUE,
     bounded = TRUE,
-    approximate_SEs=FALSE
+    approximate_SEs=FALSE,
+    criterion = "ebic.5",
+    beta_min = c("numerical", "theoretical")
     # cholesky_start # If TRUE, a model is formed with Cholesky decompositions first which is run for obtaining starting values.
 ){
   # cholesky_start <- FALSE
@@ -65,6 +67,27 @@ runmodel <- function(
   }
   
   optimizer <- x@optimizer
+
+  # Force proximal gradient for PML/PFIML estimator:
+  is_penalized <- x@estimator %in% c("PML", "PFIML")
+  if (is_penalized) {
+    experimentalWarning(x@estimator)
+    optimizer <- "proximal_gradient"
+
+    # Auto-select lambda if any penalty_lambda values are NA:
+    if (hasAutoLambda(x)) {
+      if (verbose) message("Auto-selecting lambda via grid search...")
+      x <- find_penalized_lambda(x, criterion = criterion, beta_min = beta_min,
+                                    verbose = verbose)
+      if (log) {
+        x <- addLog(x, paste0("Auto-selected lambda = ",
+                               formatC(x@penalty$lambda, format = "e", digits = 3),
+                               " via ", criterion))
+      }
+      return(x)
+    }
+  }
+
   # Default:
   if (optimizer == "default"){
     # if (x@model %in% c("varcov","lvm") && any(grepl("omega",x@parameters$matrix))){
@@ -449,43 +472,189 @@ runmodel <- function(
   # Form optimizer arguments:
   
   
-  if (grepl("cpp",optimizer)){
-    
+  if (optimizer == "proximal_gradient") {
+    # Proximal gradient (ISTA) optimizer for PML estimation:
     suppressWarnings({
       tryres <- try({
-        x <- psychonetrics_optimizer(x, lower, upper, gsub("cpp_","",optimizer), bounded)
-      }, silent = TRUE) 
+        x <- psychonetrics_proximal_gradient(x, lower, upper, bounded)
+      }, silent = TRUE)
     })
-    
-    if (is(tryres,"try-error") && !any(is.na(parVector(x)))){
-      
+
+    if (is(tryres, "try-error")) {
+      # Emergency recovery with different starting values:
       suppressWarnings({
         tryres2 <- try({
-          # browser()
           x <- updateModel(oldstart, x)
-          x <- psychonetrics_optimizer(emergencystart(x), lower, upper, gsub("cpp_","",optimizer))
-        }, silent = TRUE)    
+          x <- psychonetrics_proximal_gradient(emergencystart(x), lower, upper, bounded)
+        }, silent = TRUE)
       })
-      
-      # If still an error, break:
-      if (is(tryres2,"try-error") && !any(is.na(parVector(x)))){
-        stop(paste("Model estimation resulted in an error that could not be recovered. Try using a different optimizer with setoptimizer(...) or using different starting values. Error:\n\n",tryres2))
-        
+
+      if (is(tryres2, "try-error")) {
+        stop(paste("Proximal gradient optimization failed. Try different starting values or a smaller lambda. Error:\n\n", tryres2))
       }
-      
     }
-    
-    
+
     x@objective <- x@optim$value
-    
+    x <- updateModel(parVector(x), x, updateMatrices = TRUE)
+
+  } else if (grepl("cpp",optimizer) || optimizer == "LBFGS++"){
+
+    if (optimizer == "LBFGS++") {
+      experimentalWarning("LBFGS++ optimizer")
+      # LBFGS++ pure C++ optimizer:
+      suppressWarnings({
+        tryres <- try({
+          x <- psychonetrics_lbfgsb_optimizer(x, lower, upper, bounded)
+        }, silent = TRUE)
+      })
+
+      if (is(tryres,"try-error") && !any(is.na(parVector(x)))){
+        suppressWarnings({
+          tryres2 <- try({
+            x <- updateModel(oldstart, x)
+            x <- psychonetrics_lbfgsb_optimizer(emergencystart(x), lower, upper, bounded)
+          }, silent = TRUE)
+        })
+
+        if (is(tryres2,"try-error") && !any(is.na(parVector(x)))){
+          stop(paste("Model estimation resulted in an error that could not be recovered. Try using a different optimizer with setoptimizer(...) or using different starting values. Error:\n\n",tryres2))
+        }
+      }
+    } else {
+      # roptim-based C++ optimizers:
+      suppressWarnings({
+        tryres <- try({
+          x <- psychonetrics_optimizer(x, lower, upper, gsub("cpp_","",optimizer), bounded)
+        }, silent = TRUE)
+      })
+
+      if (is(tryres,"try-error") && !any(is.na(parVector(x)))){
+
+        suppressWarnings({
+          tryres2 <- try({
+            # browser()
+            x <- updateModel(oldstart, x)
+            x <- psychonetrics_optimizer(emergencystart(x), lower, upper, gsub("cpp_","",optimizer))
+          }, silent = TRUE)
+        })
+
+        # If still an error, break:
+        if (is(tryres2,"try-error") && !any(is.na(parVector(x)))){
+          stop(paste("Model estimation resulted in an error that could not be recovered. Try using a different optimizer with setoptimizer(...) or using different starting values. Error:\n\n",tryres2))
+        }
+
+      }
+    } # end else (roptim-based)
+
+
+    x@objective <- x@optim$value
+
     x <- updateModel(parVector(x),x,updateMatrices = TRUE) # FIXME: Move this to C++!
-    
-    
-    
-    
-    
-    
-    
+
+  } else if (grepl("nloptr", optimizer)) {
+    # nloptr-based optimizers (truncated Newton etc.)
+
+    # Build closure wrappers that capture the model:
+    model_for_nloptr <- x
+    if (x@cpp) {
+      fn_nloptr <- function(par) psychonetrics_fitfunction_cpp(par, model_for_nloptr)
+      gr_nloptr <- function(par) psychonetrics_gradient_cpp(par, model_for_nloptr)
+    } else {
+      fn_nloptr <- function(par) psychonetrics_fitfunction(par, model_for_nloptr)
+      gr_nloptr <- function(par) psychonetrics_gradient(par, model_for_nloptr)
+    }
+
+    # Set default control options:
+    # Key insight: truncated Newton has an internal gradient-norm convergence
+    # criterion that fires as NLOPT_SUCCESS before xtol/ftol checks. With tight
+    # ftol_rel (e.g. 1.49e-8) the algorithm over-converges, driving gradients to
+    # ~1e-13 where numeric Jacobian checks fail due to floating-point noise.
+    # Setting ftol_rel=1e-5 ensures ftol stops the optimizer at a reasonable
+    # point (gradient ~1e-6, similar to nlminb), with good Jacobian correlation.
+    nloptr_control <- list(
+      xtol_rel = 1.5e-8,
+      ftol_rel = 1e-5,
+      ftol_abs = 0,
+      maxeval  = 20000L
+    )
+
+    # Allow user to override via optim.args:
+    if (!is.null(optim.control$control)) {
+      nloptr_control <- modifyList(nloptr_control, optim.control$control)
+    }
+
+    # Set bounds (NULL if unbounded):
+    nloptr_lower <- if (bounded) lower else NULL
+    nloptr_upper <- if (bounded) upper else NULL
+
+    # Run optimizer with error recovery:
+    suppressWarnings({
+      tryres <- try({
+        nloptr_out <- tnewton(
+          x0      = start,
+          fn      = fn_nloptr,
+          gr      = gr_nloptr,
+          lower   = nloptr_lower,
+          upper   = nloptr_upper,
+          precond = TRUE,
+          restart = TRUE,
+          control = nloptr_control
+        )
+      }, silent = TRUE)
+    })
+
+    if (is(tryres, "try-error") || any(is.na(nloptr_out$par))) {
+      # Emergency recovery with different starting values:
+      x <- updateModel(oldstart, x)
+      model_for_nloptr <- emergencystart(x)
+      estart <- parVector(model_for_nloptr)
+
+      # Rebuild closures with emergency model:
+      if (x@cpp) {
+        fn_nloptr <- function(par) psychonetrics_fitfunction_cpp(par, model_for_nloptr)
+        gr_nloptr <- function(par) psychonetrics_gradient_cpp(par, model_for_nloptr)
+      } else {
+        fn_nloptr <- function(par) psychonetrics_fitfunction(par, model_for_nloptr)
+        gr_nloptr <- function(par) psychonetrics_gradient(par, model_for_nloptr)
+      }
+
+      suppressWarnings({
+        tryres2 <- try({
+          nloptr_out <- tnewton(
+            x0      = estart,
+            fn      = fn_nloptr,
+            gr      = gr_nloptr,
+            lower   = nloptr_lower,
+            upper   = nloptr_upper,
+            precond = TRUE,
+            restart = TRUE,
+            control = nloptr_control
+          )
+        }, silent = TRUE)
+      })
+
+      if (is(tryres2, "try-error") || any(is.na(nloptr_out$par))) {
+        stop(paste("Model estimation resulted in an error that could not be recovered. Try using a different optimizer with setoptimizer(...) or using different starting values. Error:\n\n", tryres2))
+      }
+    }
+
+    # Standardize output to match psychonetrics @optim format:
+    # nloptr convergence: positive (1-4) = success, 5 = maxeval, negative = error
+    nloptr_convergence <- if (nloptr_out$convergence > 0 && nloptr_out$convergence < 5) 0L else nloptr_out$convergence
+    optimresults <- list(
+      par         = nloptr_out$par,
+      value       = nloptr_out$value,
+      convergence = nloptr_convergence,
+      message     = nloptr_out$message,
+      fncount     = nloptr_out$iter,
+      grcount     = nloptr_out$iter,
+      optimizer   = optimizer
+    )
+    class(optimresults) <- c("list", class(optimresults))
+    x@optim <- optimresults
+    x@objective <- optimresults$value
+    x <- updateModel(nloptr_out$par, x, updateMatrices = TRUE)
+
   } else {
     optim.control$par <- start
     if (x@cpp){
@@ -683,7 +852,7 @@ runmodel <- function(
   
   # Add information:
   # if (!is.null(x@fitfunctions$information)){
-  if (addInformation){
+  if (addInformation && !is_penalized){
     if (verbose){
       message("Computing Fisher information...")
     }
@@ -818,16 +987,16 @@ runmodel <- function(
     x@baseline_saturated$saturated <- x
   }
   
-  # Add fit:
-  if (addfit){
+  # Add fit (skip for penalized models - chi-square fit not meaningful):
+  if (addfit && !is_penalized){
     x <- addfit(x, verbose=verbose)
   }
-  # Add MIs:
-  if (addMIs){
-    x <- addMIs(x,analyticFisher=analyticFisher, verbose = verbose) 
+  # Add MIs (skip for penalized models):
+  if (addMIs && !is_penalized){
+    x <- addMIs(x,analyticFisher=analyticFisher, verbose = verbose)
   }
-  # Add SEs:
-  if (addSEs){
+  # Add SEs (skip for penalized models - use refit() for post-selection inference):
+  if (addSEs && !is_penalized){
     if (x@cpp){
       if (verbose){
         message("Adding standard errors...")
@@ -846,16 +1015,16 @@ runmodel <- function(
     x <- addLog(x, "Evaluated model")    
   }
   
-  # Warn about the gradient?
-  if (warn_gradient){
-    
+  # Warn about the gradient? (skip for penalized models - subgradient at zeros is non-zero)
+  if (warn_gradient && !is_penalized){
+
     if (x@cpp){
       grad <- psychonetrics_gradient_cpp(parVector(x),x)
     } else {
       grad <- psychonetrics_gradient(parVector(x),x)
     }
-    
-    
+
+
     if (mean(abs(grad)) > 1){
       warning("Model might not have converged properly: mean(abs(gradient)) > 1.")
     }
