@@ -29,24 +29,31 @@ parameters <- function(x){
   # filter only non-zero parameters and select only relevant columns:
   # FIXME: std not yet implemented, so remove now:
   
-  if (!all(is.na(x@parameters$se_boot))){
+  if (x@estimator == "PML") {
+    cols <- c("var1","op","var2","est","penalty_lambda","matrix","row","col","group","par")
+    boots <- FALSE
+  } else if (!all(is.na(x@parameters$se_boot))){
     cols <- c("var1","op","var2","est","se","p","se_boot","p_boot","matrix","row","col","group","par")
     boots <- TRUE
   } else {
     cols <- c("var1","op","var2","est","se","p","matrix","row","col","group","par")
     boots <- FALSE
   }
-  parTable <- parTable %>% filter(!.data[['fixed']]|.data[['est']] != 0) %>% 
+  parTable <- parTable %>% filter(drop(!.data[['fixed']]|.data[['est']] != 0)) %>%
     select(all_of(cols))
-  
+
   # Make var2 nicer:
   parTable$var2 <- ifelse(is.na(parTable$var2),"",parTable$var2)
-  
+
   # Pretty numbers:
   parTable$est <- goodNum2(parTable$est)
-  parTable$se <- goodNum(parTable$se)
-  parTable$p <- goodNum(parTable$p)
-  
+  if (x@estimator == "PML") {
+    parTable$penalty_lambda <- goodNum2(parTable$penalty_lambda)
+  } else {
+    parTable$se <- goodNum(parTable$se)
+    parTable$p <- goodNum(parTable$p)
+  }
+
   if (boots){
     parTable$se_boot <- goodNum(parTable$se_boot)
     parTable$p_boot <- goodNum(parTable$p_boot)
