@@ -14,9 +14,13 @@
 #include "18_dlvm1_derivatives_cpp.h"
 #include "19_tsdlvm1_derivatives_cpp.h"
 #include "20_meta_varcov_derivatives_cpp.h"
+#include "23_meta_lvm_derivatives_cpp.h"
+#include "24_meta_var1_derivatives_cpp.h"
 #include "21_Ising_derivatives_cpp.h"
 #include "22_ml_lvm_derivatives_cpp.h"
 #include "02_algebrahelpers_modelMatrix_cpp.h"
+#include "04_generalfit_optimWorkspace.h"
+#include "09_PenMLestimator_penalty_helpers_cpp.h"
 
 // [[Rcpp::depends(RcppArmadillo)]]
 using namespace Rcpp;
@@ -68,8 +72,9 @@ void psychonetrics_gradient_cpp_inner(
   // Prepare model:
   Rcpp::List prep = prepareModel_cpp(x, model);
   
-  // Manual part:
-  arma::sp_mat manualPart = Mmatrix_cpp_list(model.slot("parameters"));
+  // Manual part (cached in workspace):
+  const OptimWorkspace& ws = getOrBuildWorkspace(model);
+  const arma::sp_mat& manualPart = ws.Mmatrix;
   
   
   // What estimator:
@@ -85,27 +90,31 @@ void psychonetrics_gradient_cpp_inner(
   // Estimator part:
   arma::mat estimatorPart;
   
-  if (estimator == "ML"){
+  if (estimator == "ML" || estimator == "PML"){
     if (distribution == "Gaussian"){
-      
+
       estimatorPart = jacobian_gaussian_sigma_cpp(prep);
-      
+
     } else if (distribution == "Ising"){
-      
+
       estimatorPart = jacobian_Ising_cpp(prep);
-      
+
     } else {
       Rf_error("Distribution not supported for ML estimator.");
     }
-    
+
   } else if (estimator == "ULS" || estimator == "WLS" || estimator == "DWLS"){
     
     estimatorPart = ULS_gradient_Gauss_cpp(prep);
     
-  } else if (estimator == "FIML"){
-    
+  } else if (estimator == "PFIML"){
+
     estimatorPart = jacobian_fiml_gaussian_sigma_cpp(prep);
-    
+
+  } else if (estimator == "FIML"){
+
+    estimatorPart = jacobian_fiml_gaussian_sigma_cpp(prep);
+
   } else {
     Rf_error("Estimator not supported.");
   }
@@ -142,19 +151,26 @@ void psychonetrics_gradient_cpp_inner(
     modelPart = d_phi_theta_Ising_cpp(prep);
     
   }  else if (usemodel == "ml_lvm"){
-    
+
     modelPart = d_phi_theta_ml_lvm_cpp(prep);
-    
+
+  }  else if (usemodel == "meta_lvm"){
+
+    modelPart = d_phi_theta_meta_lvm_cpp(prep);
+
+  }  else if (usemodel == "meta_var1"){
+
+    modelPart = d_phi_theta_meta_var1_cpp(prep);
+
   }
-  
+
   // Compute the gradient
   // FIXME? Use sparse?
   
-  // Get parameter table:
-  Rcpp::List pars = model.slot("parameters");
-  arma::vec parnum = pars["par"];
-  int freePar = max(parnum);
-  
+  // Parameter info (cached in workspace):
+  const arma::vec& parnum = ws.parnum;
+  int freePar = ws.nFreePar;
+
   arma::mat Jac(1,freePar);
   
   
@@ -191,18 +207,12 @@ void psychonetrics_gradient_cpp_inner(
     }
     
     
-    // Get parameter table:
-    Rcpp::List pars = model.slot("parameters");
-    arma::vec parnum = pars["par"];
-    
+    // Parameter info (already cached in ws above)
     // ints to use:
     int i;
-    
-    // Number of free parameters:
-    // int freePar = max(parnum);
-    
+
     // Number of total parameters:
-    int totalPar = parnum.n_elem;
+    int totalPar = ws.nParTotal;
     
     // Empty Fisher:
     Jac.fill(0);
@@ -222,7 +232,11 @@ void psychonetrics_gradient_cpp_inner(
   
   
   grad = vectorise(Jac);
-  // return(vectorise(Jac));
+
+  // Add penalty gradient for PML/PFIML estimator:
+  if (ws.estimator == "PML" || ws.estimator == "PFIML") {
+    addPenaltyGradient_cpp(grad, x, ws);
+  }
 }
 
 
