@@ -7,48 +7,40 @@
 #include "02_algebrahelpers_RcppHelpers.h"
 #include "03_modelformation_formModelMatrices_cpp.h"
 #include "03_modelformation_impliedcovstructures.h"
+#include "04_generalfit_optimWorkspace.h"
 
 // [[Rcpp::depends(RcppArmadillo)]]
 using namespace Rcpp;
 using namespace arma;
 
-// [[Rcpp::export]]
-Rcpp::List implied_dlvm1_cpp(
+// Core implementation that takes pre-formed model matrices
+Rcpp::List implied_dlvm1_cpp_core(
+    Rcpp::List x,
     const S4& model,
     bool all = false
 ){
   int i, t, tt;
-  
 
-  
-  
-  S4 sample = model.slot("sample");
-  Rcpp::List means = sample.slot("means");
-  
-  
-  // Form basic model matrices:
-  Rcpp::List x = formModelMatrices_cpp(model);
-  
+  // Read constant data from cached workspace:
+  const OptimWorkspace& ws = getOrBuildWorkspace(model);
+  const Rcpp::List& types = ws.types;
 
-  // Types:
-  Rcpp::List types = model.slot("types");
-  
   std::string within_latent = types["within_latent"];
   std::string within_residual = types["within_residual"];
   std::string between_latent = types["between_latent"];
   std::string between_residual = types["between_residual"];
-  
+
   // Add implied cov structure:
   x = impliedcovstructures_cpp(x, "zeta_within", within_latent, all);
   x = impliedcovstructures_cpp(x, "epsilon_within", within_residual, all);
   x = impliedcovstructures_cpp(x, "zeta_between", between_latent, all);
   x = impliedcovstructures_cpp(x, "epsilon_between", between_residual, all);
-  
+
   int nGroup = x.length();
   int g;
-  
+
   // General stuff:
-  Rcpp::List extramats = model.slot("extramatrices");
+  const Rcpp::List& extramats = ws.extramatrices;
   arma::mat design = extramats["design"];
   
   int nVar = design.n_rows;
@@ -204,3 +196,15 @@ Rcpp::List implied_dlvm1_cpp(
   return(x);
 }
 
+// Original version: forms matrices from the S4 model
+// [[Rcpp::export]]
+Rcpp::List implied_dlvm1_cpp(
+    const S4& model,
+    bool all = false
+){
+  // Form basic model matrices:
+  Rcpp::List x = formModelMatrices_cpp(model);
+
+  return implied_dlvm1_cpp_core(x, model, all);
+}
+
