@@ -6,8 +6,8 @@
 #include "02_algebragelpers_kronecker.h"
 #include "02_algebrahelpers_RcppHelpers.h"
 #include "03_modelformation_formModelMatrices_cpp.h"
+#include "04_generalfit_optimWorkspace.h"
 #include "03_modelformation_impliedcovstructures.h"
-#include "b_modelexpansion_updateModel_cpp.h"
 #include "18_dlvm1_implied_cpp.h"
 
 // [[Rcpp::depends(RcppArmadillo)]]
@@ -21,47 +21,25 @@ Rcpp::List prepare_dlvm1_cpp(
 ){
   int g, i;
   
-  // New model:
-  S4 newMod = updateModel_cpp(x,model,false);
-  
-  // sample slot:
-  S4 sample = newMod.slot("sample");
-  
-  // Things needed:
-  bool corinput = sample.slot("corinput");
-  bool meanstructure = newMod.slot("meanstructure");
-  
-  // Groups table:
-  Rcpp::List group = sample.slot("groups");
-  
-  // Variables table:
-  Rcpp::List variables = sample.slot("variables");
-  arma::vec vars = variables["id"];
-  int nVar = vars.n_elem;
-  
-  // Number of groups:
-  arma::vec id = group["id"];
-  int nGroup = id.n_elem;
-  
-  // Number of observations per group:
-  arma::vec nPerGroup = group["nobs"];
-  
-  // Compute implied matrices:
-  Rcpp::List imp = implied_dlvm1_cpp(newMod, false);
-  
-  // Extra matrices:
-  Rcpp::List extramatrices = model.slot("extramatrices");
-  
-  // Types:
-  Rcpp::List types = model.slot("types");
-  
-  // Total sample:
-  double nTotal = sum(nPerGroup);
-        
-  // Sample stats:
-  Rcpp::List S = sample.slot("covs");
-  Rcpp::List means =  sample.slot("means");
-  Rcpp::List thresholds = sample.slot("thresholds");
+  // Form model matrices using cached workspace:
+  const OptimWorkspace& ws = getOrBuildWorkspace(model);
+  Rcpp::List mats = formModelMatrices_direct(x, ws.mapping);
+
+  // Compute implied matrices using core function:
+  Rcpp::List imp = implied_dlvm1_cpp_core(mats, model, false);
+
+  // Read constant data from cached workspace (no S4 slot reads):
+  bool corinput = ws.corinput;
+  bool meanstructure = ws.meanstructure;
+  int nVar = ws.nVar;
+  int nGroup = ws.nGroup;
+  arma::vec nPerGroup = ws.nPerGroup;
+  const Rcpp::List& extramatrices = ws.extramatrices;
+  const Rcpp::List& types = ws.types;
+  double nTotal = ws.nTotal;
+  const Rcpp::List& S = ws.sampleCovs;
+  const Rcpp::List& means = ws.sampleMeans;
+  const Rcpp::List& thresholds = ws.sampleThresholds;
 
   
   // Group models:
